# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, SequenceNotStr, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.api_gateway.expression_template import fallthrough_create_params
from ....types.api_gateway.expression_template.fallthrough_create_response import FallthroughCreateResponse

__all__ = ["FallthroughResource", "AsyncFallthroughResource"]


class FallthroughResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FallthroughResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FallthroughResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FallthroughResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FallthroughResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        hosts: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FallthroughCreateResponse:
        """Creates an expression template fallthrough rule for API Shield.

        Used for
        configuring default behavior when no other expression templates match.

        Args:
          zone_id: Identifier.

          hosts: List of hosts to be targeted in the expression

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/api_gateway/expression-template/fallthrough", zone_id=zone_id),
            body=maybe_transform({"hosts": hosts}, fallthrough_create_params.FallthroughCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FallthroughCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[FallthroughCreateResponse], ResultWrapper[FallthroughCreateResponse]),
        )


class AsyncFallthroughResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFallthroughResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFallthroughResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFallthroughResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFallthroughResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        hosts: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FallthroughCreateResponse:
        """Creates an expression template fallthrough rule for API Shield.

        Used for
        configuring default behavior when no other expression templates match.

        Args:
          zone_id: Identifier.

          hosts: List of hosts to be targeted in the expression

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/api_gateway/expression-template/fallthrough", zone_id=zone_id),
            body=await async_maybe_transform({"hosts": hosts}, fallthrough_create_params.FallthroughCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FallthroughCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[FallthroughCreateResponse], ResultWrapper[FallthroughCreateResponse]),
        )


class FallthroughResourceWithRawResponse:
    def __init__(self, fallthrough: FallthroughResource) -> None:
        self._fallthrough = fallthrough

        self.create = to_raw_response_wrapper(
            fallthrough.create,
        )


class AsyncFallthroughResourceWithRawResponse:
    def __init__(self, fallthrough: AsyncFallthroughResource) -> None:
        self._fallthrough = fallthrough

        self.create = async_to_raw_response_wrapper(
            fallthrough.create,
        )


class FallthroughResourceWithStreamingResponse:
    def __init__(self, fallthrough: FallthroughResource) -> None:
        self._fallthrough = fallthrough

        self.create = to_streamed_response_wrapper(
            fallthrough.create,
        )


class AsyncFallthroughResourceWithStreamingResponse:
    def __init__(self, fallthrough: AsyncFallthroughResource) -> None:
        self._fallthrough = fallthrough

        self.create = async_to_streamed_response_wrapper(
            fallthrough.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .dex import (
    DEXResource,
    AsyncDEXResource,
    DEXResourceWithRawResponse,
    AsyncDEXResourceWithRawResponse,
    DEXResourceWithStreamingResponse,
    AsyncDEXResourceWithStreamingResponse,
)
from .colos import (
    ColosResource,
    AsyncColosResource,
    ColosResourceWithRawResponse,
    AsyncColosResourceWithRawResponse,
    ColosResourceWithStreamingResponse,
    AsyncColosResourceWithStreamingResponse,
)
from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .tests import (
    TestsResource,
    AsyncTestsResource,
    TestsResourceWithRawResponse,
    AsyncTestsResourceWithRawResponse,
    TestsResourceWithStreamingResponse,
    AsyncTestsResourceWithStreamingResponse,
)
from .commands import (
    CommandsResource,
    AsyncCommandsResource,
    CommandsResourceWithRawResponse,
    AsyncCommandsResourceWithRawResponse,
    CommandsResourceWithStreamingResponse,
    AsyncCommandsResourceWithStreamingResponse,
)
from .http_tests import (
    HTTPTestsResource,
    AsyncHTTPTestsResource,
    HTTPTestsResourceWithRawResponse,
    AsyncHTTPTestsResourceWithRawResponse,
    HTTPTestsResourceWithStreamingResponse,
    AsyncHTTPTestsResourceWithStreamingResponse,
)
from .fleet_status import (
    FleetStatusResource,
    AsyncFleetStatusResource,
    FleetStatusResourceWithRawResponse,
    AsyncFleetStatusResourceWithRawResponse,
    FleetStatusResourceWithStreamingResponse,
    AsyncFleetStatusResourceWithStreamingResponse,
)
from .traceroute_tests import (
    TracerouteTestsResource,
    AsyncTracerouteTestsResource,
    TracerouteTestsResourceWithRawResponse,
    AsyncTracerouteTestsResourceWithRawResponse,
    TracerouteTestsResourceWithStreamingResponse,
    AsyncTracerouteTestsResourceWithStreamingResponse,
)
from .warp_change_events import (
    WARPChangeEventsResource,
    AsyncWARPChangeEventsResource,
    WARPChangeEventsResourceWithRawResponse,
    AsyncWARPChangeEventsResourceWithRawResponse,
    WARPChangeEventsResourceWithStreamingResponse,
    AsyncWARPChangeEventsResourceWithStreamingResponse,
)
from .traceroute_test_results import (
    TracerouteTestResultsResource,
    AsyncTracerouteTestResultsResource,
    TracerouteTestResultsResourceWithRawResponse,
    AsyncTracerouteTestResultsResourceWithRawResponse,
    TracerouteTestResultsResourceWithStreamingResponse,
    AsyncTracerouteTestResultsResourceWithStreamingResponse,
)

__all__ = [
    "WARPChangeEventsResource",
    "AsyncWARPChangeEventsResource",
    "WARPChangeEventsResourceWithRawResponse",
    "AsyncWARPChangeEventsResourceWithRawResponse",
    "WARPChangeEventsResourceWithStreamingResponse",
    "AsyncWARPChangeEventsResourceWithStreamingResponse",
    "CommandsResource",
    "AsyncCommandsResource",
    "CommandsResourceWithRawResponse",
    "AsyncCommandsResourceWithRawResponse",
    "CommandsResourceWithStreamingResponse",
    "AsyncCommandsResourceWithStreamingResponse",
    "ColosResource",
    "AsyncColosResource",
    "ColosResourceWithRawResponse",
    "AsyncColosResourceWithRawResponse",
    "ColosResourceWithStreamingResponse",
    "AsyncColosResourceWithStreamingResponse",
    "FleetStatusResource",
    "AsyncFleetStatusResource",
    "FleetStatusResourceWithRawResponse",
    "AsyncFleetStatusResourceWithRawResponse",
    "FleetStatusResourceWithStreamingResponse",
    "AsyncFleetStatusResourceWithStreamingResponse",
    "HTTPTestsResource",
    "AsyncHTTPTestsResource",
    "HTTPTestsResourceWithRawResponse",
    "AsyncHTTPTestsResourceWithRawResponse",
    "HTTPTestsResourceWithStreamingResponse",
    "AsyncHTTPTestsResourceWithStreamingResponse",
    "TestsResource",
    "AsyncTestsResource",
    "TestsResourceWithRawResponse",
    "AsyncTestsResourceWithRawResponse",
    "TestsResourceWithStreamingResponse",
    "AsyncTestsResourceWithStreamingResponse",
    "TracerouteTestResultsResource",
    "AsyncTracerouteTestResultsResource",
    "TracerouteTestResultsResourceWithRawResponse",
    "AsyncTracerouteTestResultsResourceWithRawResponse",
    "TracerouteTestResultsResourceWithStreamingResponse",
    "AsyncTracerouteTestResultsResourceWithStreamingResponse",
    "TracerouteTestsResource",
    "AsyncTracerouteTestsResource",
    "TracerouteTestsResourceWithRawResponse",
    "AsyncTracerouteTestsResourceWithRawResponse",
    "TracerouteTestsResourceWithStreamingResponse",
    "AsyncTracerouteTestsResourceWithStreamingResponse",
    "RulesResource",
    "AsyncRulesResource",
    "RulesResourceWithRawResponse",
    "AsyncRulesResourceWithRawResponse",
    "RulesResourceWithStreamingResponse",
    "AsyncRulesResourceWithStreamingResponse",
    "DEXResource",
    "AsyncDEXResource",
    "DEXResourceWithRawResponse",
    "AsyncDEXResourceWithRawResponse",
    "DEXResourceWithStreamingResponse",
    "AsyncDEXResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .image import Image as Image
from .v1_edit_params import V1EditParams as V1EditParams
from .v1_list_params import V1ListParams as V1ListParams
from .v2_list_params import V2ListParams as V2ListParams
from .v1_create_params import V1CreateParams as V1CreateParams
from .v1_list_response import V1ListResponse as V1ListResponse
from .v2_list_response import V2ListResponse as V2ListResponse
from .v1_delete_response import V1DeleteResponse as V1DeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["PolicyTestGetResponse"]


class PolicyTestGetResponse(BaseModel):
    id: Optional[str] = None
    """The UUID of the policy test."""

    percent_approved: Optional[int] = None
    """
    The percentage of (processed) users approved based on policy evaluation results.
    """

    percent_blocked: Optional[int] = None
    """The percentage of (processed) users blocked based on policy evaluation results."""

    percent_errored: Optional[int] = None
    """The percentage of (processed) users errored based on policy evaluation results."""

    percent_users_processed: Optional[int] = None
    """The percentage of users processed so far (of the entire user base)."""

    status: Optional[Literal["blocked", "processing", "exceeded time", "complete"]] = None
    """The status of the policy test."""

    total_users: Optional[int] = None
    """The total number of users in the user base."""

    users_approved: Optional[int] = None
    """The number of (processed) users approved based on policy evaluation results."""

    users_blocked: Optional[int] = None
    """The number of (processed) users blocked based on policy evaluation results."""

    users_errored: Optional[int] = None
    """The number of (processed) users errored based on policy evaluation results."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .audit import (
    AuditResource,
    AsyncAuditResource,
    AuditResourceWithRawResponse,
    AsyncAuditResourceWithRawResponse,
    AuditResourceWithStreamingResponse,
    AsyncAuditResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["LogsResource", "AsyncLogsResource"]


class LogsResource(SyncAPIResource):
    @cached_property
    def audit(self) -> AuditResource:
        return AuditResource(self._client)

    @cached_property
    def with_raw_response(self) -> LogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LogsResourceWithStreamingResponse(self)


class AsyncLogsResource(AsyncAPIResource):
    @cached_property
    def audit(self) -> AsyncAuditResource:
        return AsyncAuditResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLogsResourceWithStreamingResponse(self)


class LogsResourceWithRawResponse:
    def __init__(self, logs: LogsResource) -> None:
        self._logs = logs

    @cached_property
    def audit(self) -> AuditResourceWithRawResponse:
        return AuditResourceWithRawResponse(self._logs.audit)


class AsyncLogsResourceWithRawResponse:
    def __init__(self, logs: AsyncLogsResource) -> None:
        self._logs = logs

    @cached_property
    def audit(self) -> AsyncAuditResourceWithRawResponse:
        return AsyncAuditResourceWithRawResponse(self._logs.audit)


class LogsResourceWithStreamingResponse:
    def __init__(self, logs: LogsResource) -> None:
        self._logs = logs

    @cached_property
    def audit(self) -> AuditResourceWithStreamingResponse:
        return AuditResourceWithStreamingResponse(self._logs.audit)


class AsyncLogsResourceWithStreamingResponse:
    def __init__(self, logs: AsyncLogsResource) -> None:
        self._logs = logs

    @cached_property
    def audit(self) -> AsyncAuditResourceWithStreamingResponse:
        return AsyncAuditResourceWithStreamingResponse(self._logs.audit)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["IndexDimensionConfiguration"]


class IndexDimensionConfiguration(BaseModel):
    dimensions: int
    """Specifies the number of dimensions for the index"""

    metric: Literal["cosine", "euclidean", "dot-product"]
    """Specifies the type of metric to use calculating distance."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["AssetCreateResponse"]


class AssetCreateResponse(BaseModel):
    id: int
    """Asset ID."""

    name: str
    """Asset name."""

    created: Optional[datetime] = None
    """Defines the asset creation time."""

    description: Optional[str] = None
    """Asset description."""

    file_type: Optional[str] = None
    """Asset file type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["Dimension"]

Dimension: TypeAlias = Literal["event", "appID", "coloName", "ipVersion"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["OktaGroupRule", "Okta"]


class Okta(BaseModel):
    identity_provider_id: str
    """The ID of your Okta identity provider."""

    name: str
    """The name of the Okta group."""


class OktaGroupRule(BaseModel):
    """
    Matches an Okta group.
    Requires an Okta identity provider.
    """

    okta: Okta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["ConfigGetResponse"]


class ConfigGetResponse(BaseModel):
    amount: Optional[float] = None

    disabled_reason: Optional[str] = FieldInfo(alias="disabledReason", default=None)

    error: Optional[str] = None

    last_failed_at: Optional[float] = FieldInfo(alias="lastFailedAt", default=None)

    threshold: Optional[float] = None
